{% extends "yacon/nexus_base.html" %}

{% block scripts %}
    {{ block.super }}
    <script type="text/JavaScript">
        $(document).ready(function(){
            $('#navbar').tabnav('select_by_id', 'nav_users');

            {% if clear_autocomplete %}
                // don't want to disable autocomplete, but don't want the
                // fields pre-filled either
                $('form input:text').val('');
            {% endif %}
        });
    </script>
{% endblock scripts %}

{% block content %}
<h2>{{title}}</h2>
<form action="" method="post">
    {% csrf_token %}
    <table border="0">
        {{ form.as_table }}
        <tr>
            <td> <br/> </td>
            <td align="right"> 
                <input type="button"
                    onClick="self.location='/yacon/nexus/users/list_users/'" 
                    value="Cancel"/>
                &nbsp; &nbsp;
                <input type="submit" value="Save"/> 
            </td>
        </tr>
    </table>
</form>
{% endblock content %}
