{% extends "yacon/examples/base.html" %}
{% load yacon_tags %}

{% block body %}
<h1> {% editable_page_title page %} </h1>

<h2> Content </h2>

{% editable_block_by_key "blurb" %}

<h2> Poll </h2>

{% block_by_key "poll" %}

<h2> Dynamic </h2>

<p>
{% dynamic_content_by_key "right_now" %}
</p>

<h2> Menu </h2>

{% menu "Menu 1" %}

<h2> Last Updated </h2>

<p>
    {% page_last_updated page %}
</p>

{% endblock body %}


