{% extends "yacon/examples/base.html" %}


{% block body %}
<h1> Upload Examples </h1>

<ul>
    <li> <a href="/yacon/examples/uploads/simple.html">Simple</a>
    <li> <a href="/yacon/examples/uploads/user.html">User</a>
    <li> <a href="/yacon/examples/uploads/image.html">Image</a>
</ul>
{% endblock body %}
