<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" 
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
<head>
    <title>File Browser</title>
    <meta http-equiv="CACHE-CONTROL" CONTENT="NO-CACHE">
    {% if settings.YACON_USE_LOCAL_JQUERY %}
        <script type="text/JavaScript" 
          src="/static/yacon/js/jquery-1.7.2.min.js"></script>
    {% else %}
        <script type="text/JavaScript"
            src="http://ajax.googleapis.com/ajax/libs/jquery/1.7/jquery.min.js">
        </script>
    {% endif %}
    <script type="text/JavaScript" 
        src="/static/yacon/js/jquery-ui-1.8.19.custom.min.js">
    </script>
    <link type="text/css" rel="stylesheet" 
        href="/static/yacon/css/smoothness/jquery-ui-1.8.19.custom.css" />	

    <script type="text/JavaScript" src="/static/yacon/js/yacon/utils.js">
    </script>

    {% block css %}
    {% endblock css %}
    {% block scripts %}
    {% endblock scripts %}

    <link rel="stylesheet" type="text/css" 
      href="/static/yacon/css/browser.css"/>
    <link rel="stylesheet" type="text/css" 
        href="/static/yacon/css/browser_popup.css"/>
</head>
<body>
    <div id="content">
        {% block content %}
        {% endblock content %}
    </div>
</body>
</html>
